/*
 * File:   main.c
 * Author: Manolo Serrano
 * http://tren.enmicasa.net
 *
 * Created on 12 de marzo de 2013, 10:11
 *
 *  This file is part of rpic.
 *
 *  rpic is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  rpic is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with rpic.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Main Routine
 *
 */

/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#elif defined(__18CXX)
#include <p18cxxx.h>   /* C18 General Include File */
#endif

#if defined(__XC) || defined(HI_TECH_C)
#include <stdint.h>        /* For uint8_t definition */
#include <stdbool.h>       /* For true/false definition */
#endif

#include "system.h"        /* System funct/params, like osc/peripheral config */
#include "user.h"          /* User funct/params, such as InitApp */
#include "nmra.h"          //NMRA
#include "mm.h"            //MM defintiions
#include "mmx.h"           //MM defintiions
#include "queue.h"         //Queue is here
#include "serial.h"        //Buffer is here and also abortsignal
#include "timer.h"         //timer is her
#include "program.h"       //program definitions

/******************************************************************************/
/* User Global Variable Declaration                                           */
/******************************************************************************/

char buffer[MAXBYTES];
bit abortsignal;
char status;
char old_status;

/******************************************************************************/
/* Main Program                                                               */

/******************************************************************************/

void main(void) {
    char result;
    char command;
    char i;
    /* Configure the oscillator for the device */
    ConfigureOscillator();

    /* Initialize I/O and Peripherals for application */
    InitApp();

    //Testing code--------------------------------------------

    //     status = CYCLEON;
    /*  bufferIN[0]=0;    //no need for this mfx test;
      bufferIN[1]=0;    //address byte 1
      bufferIN[2]=1;   //address byte 2
      bufferIN[3]=20;
      bufferIN[4]=0b10101010;
      bufferIN[5]=0b00000001;
      bufferIN[6]=0;    //no needed for this test

      locoMMX();*/
    //  di();
    /*  old_status = CYCLEON;
      status=NEWCMD;
      bufferIN[0]=0x64;    //no need for this mfx test;
      bufferIN[1]=34;    //address byte 1
      bufferIN[2]=4;   //address byte 2
      bufferIN[3]=3;
      bufferIN[4]=0xa3;*/
    // locoMM();

    //--------------------------------------------------------

    while (1) {
        while (status == CYCLEON) {
            getfromQueueNMRA(locosMessageQueue.positionNMRA);

            result = locoNMRA(); //not error handling implemented yet.. error messages only sent after reciving byte...with checksum
            //errors at this stage only responds to a corrupted queue... only good packets are stored.
            //if queue is empty.. address=0 type=EMPTY...and therefore the result=error10.. we do nto want to transmit such packet... this is treated within the loco()
            //Any other error is only possible with a corrupted queue
            //Some kind of visual feedback with a led could be implemented=iddle mode or corrupted queue
            __delay_ms(TIMEBTCYCLE);
            TRISA = TRISAout;
            idlePacket(NORMALpreambleBitsNMRA);
            TRISA = TRISAconf;
            __delay_ms(TIMEBTCYCLE);
            getfromQueueMM(locosMessageQueue.positionMM);
            if (buffer[0] < LOCOMMX) {
                result = locoMM();
                if (buffer[0] == LOCOMM2) {
                    getfromQueueMM(locosMessageQueue.positionMM + 1); //two extra packets for FX functions
                    result = locoMM();
                }
            }
            //            if (buffer[0]==LOCOMMX){
            //                locoMMX();
            //            }

            if (!abortsignal) { //if abortsingal.. we do not increase and repeat packet
                //if not abortsingal.. increment queue.. otherwise we will continue with the old count
                //potential problem if interruption happens here... but at this stage last packet has been sent..so no problem in updating
                if (locosMessageQueue.nextnewNMRA == 0) { //if nextnew=0... queue empty or full... in any case neded to go thwouth the entire queue
                    if ((locosMessageQueue.positionNMRA)<(locosMessageQueue.division - 1)) {
                        locosMessageQueue.positionNMRA++;
                    } else {
                        locosMessageQueue.positionNMRA = 0;
                    }
                } else {
                    if (locosMessageQueue.nextnewNMRA > (locosMessageQueue.positionNMRA + 1)) {
                        locosMessageQueue.positionNMRA++;
                        //if not end of queue
                    } else { //cycling the queue
                        locosMessageQueue.positionNMRA = 0;
                    }
                }

                if (buffer[0] > LOCOMM1Fx) { //increment the function group. Only works because there is not MM3 MM4 and so on
                    locosMessageQueue.positionMM++; //this also works for the alpha version of MMX the rest no affects
                    if (locosMessageQueue.messages[locosMessageQueue.positionMM].fgroup < 4) {
                        locosMessageQueue.messages[locosMessageQueue.positionMM].fgroup++;
                    } else {
                        locosMessageQueue.messages[locosMessageQueue.positionMM].fgroup = 1;
                    }
                }
                //This also works for the current version of MMX
                if (locosMessageQueue.nextnewMM == locosMessageQueue.division) {
                    if ((locosMessageQueue.positionMM)<(MAXLOCOMESSAGES - 2)) {
                        locosMessageQueue.positionMM++;
                    } else {
                        locosMessageQueue.positionMM = locosMessageQueue.division;
                    }
                } else {
                    if (locosMessageQueue.nextnewMM > (locosMessageQueue.positionMM + 1)) {
                        locosMessageQueue.positionMM++;
                    } else {
                        locosMessageQueue.positionMM = locosMessageQueue.division;
                    }
                }
            }

            if (bytesreceived == 0) //if timeout or free to receive new message
            { //if a new message has arrived.. abortsignal is false but status is newcms
                abortsignal = false; //if interruption right here...is newmessage... abortinsingal==true (as expected.. new messages break the reception
            } //always free at some point the signal.. otherwise it can hang after an interruption and never start again.
        }

        //i do not di() before checking status to avoid di() when cycle off..otherwise.. never come back from cycleoff
        if (status == NEWCMD) { //if interruption happens just here (new receptioN).. abortinsingal=true. I changege the aborting signal afert di().. New message will be lost since timeout will happen. timeout and sender will send it again
            //di(); //no interruptions when treating new commands       //so far enable rest of interruptions.. in case i want another peripherial to interrupt
            RCIE = 0; //no new meesages allowed when reading.. I need interruption for the timers in the signal.. not anymore
            abortsignal = false; //in case an interruption arrives just at the beginning if (staus==NEWCMS)
            for (i = 0; i != MAXBYTES; i++) {
                buffer[i] = bufferIN[i]; //I need two buffers.. whilest preparing bufferIN if we are in the loop get from queue changes buffer... so I need to secure buffeerIN
            }
            command = (buffer[0] >> 5)&0b00000111;
            buffer[0] = 0b00011111 & buffer[0]; //now buffer 0 is just the type
            char oldDspeed;
            if (command == SYSTEMCMD) { //SWITCH case not supported by PIC16..takes a long block
                if (buffer[0] != ERROR1) {
                    if (buffer[0] == PTON) {
                        ;
                        //code to active PT
                    } else if (buffer[0] == PTOFF) {
                        ;
                        //code to desactivate
                    }
                    //In principle in rocpi the status of PT does not affect the cycle... independant pins... in principle.. let's see how it evolves in the HW
                    old_status = buffer[0]; //can be cycleON or cycleOFF.. i stored in old_status becuaseit is going to be recovered later
                    result = OK;
                } else {
                    result = ERROR1;
                }
            }
            if (command == ACCNMRACMD) {
                result = accNMRA();
            }
            if (command == LOCNMRACMD) {
                result = locoNMRA();
            }
            if (command == LOCMMCMD) {
                oldDspeed = getMMoldDspeed();
                if ((buffer[0] == LOCOMM1)&((oldDspeed^buffer[2]) > 127)) { //oldspeed xor current speed... if change in bit 7.. xor makes b7=1...so
                    buffer[0] = LOCOMM1R;
                }
                if ((buffer[0] == LOCOMM2)&(oldDspeed == buffer[2])) {
                    //No changes in speed.. let's see what function has changed
                    char fchange = buffer[3]^getMMoldFx(); //If F0 changed=1... Fgroup=0; If F1 changed=10=2 Fgroup=1...if f2 changed=100=4 fgroup=2... f3=1000=8
                    buffer[0] = LOCOMM2Fx;
                    switch (fchange) {
                        case 1:
                            buffer[0] = LOCOMM2; //By default... change f0.. treated as speed command
                            break;
                        case 2:
                            buffer[4] = 1;
                            break;
                        case 4:
                            buffer[4] = 2;
                            break;
                        case 8:
                            buffer[4] = 3;
                            break;
                        case 16:
                            buffer[4] = 4;
                            break;
                    }
                }
                result = locoMM();
            }
            //            if (command==LOCMMXCMD){         //Alpha version
            //                    locoMMX();
            //            }
            if (command == PROGCMD) {
                program();
            }
            status = old_status;
            WriteUSART(buffer[MAXBYTES - 1] + result);
            //So far no other status defined.
            //ei();
            commandlength = 0; //ready for new packet
            RCIE = 1;         // new meesages allowed when reading.. I need interruption for the timers in the signal
        }
    }
}
