/*
 * File:   nmra.c
 * Author: Manolo Serrano
 * http://tren.enmicasa.net
 *
 * Created on 12 de marzo de 2013, 10:11
 *
 *  This file is part of rpic.
 *
 *  rpic is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  rpic is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with rpic.  If not, see <http://www.gnu.org/licenses/>.
 *
 * QUEUE Routines
 *
 */

/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */


#include "user.h"           //access to bit and byte functions
#include "queue.h"          
#include "serial.h"         //buffer is here
/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/
queue locosMessageQueue;

void initQueue(void) {
    char i;
    locosMessageQueue.nextnewNMRA = 0;
    locosMessageQueue.positionNMRA = 0;
    locosMessageQueue.division = DIVISION;
    locosMessageQueue.nextnewMM = DIVISION;
    locosMessageQueue.positionMM = DIVISION;

    for (i = 0;  i < MAXLOCOMESSAGES;i++) {
        locosMessageQueue.messages[i].address = 0;
        locosMessageQueue.messages[i].dspeed = 0;
        locosMessageQueue.messages[i].fgroup = 0;
        locosMessageQueue.messages[i].type = EMPTY;     //this way... empy messages are never process... type 0=ACCMM and ACCNMRA.. and even if they should not be in the queue... if they are, they could be process
    }
}

void updateQueueNMRA() {
    char i = 0;
    static bit found;
    static bit longadress;
    int address = 0;
    found = false;
    longadress = false;

    if ((buffer[0] >= LOCODCCN14)&(buffer[0] <= LOCODCCNfx)) {
        //short address
        address = buffer[1]&0b01111111; //without the and.. it should work...rocrail should grant that the address is<127
    }
    if ((buffer[0] >= LOCODCCL14)&(buffer[0] <= LOCODCCLfx)) {
        //long address
        //recieved (buffer(1) (buffer(2)=int
        address = buffer[2];
        byte_of(address, 1) = buffer[1]; 
        longadress = true;
    }

    while ((!found) && (i != locosMessageQueue.division)) {     //CYCLYINGqueue.. I need to check all
        if (locosMessageQueue.messages[i].type == buffer[0]) {
            if (locosMessageQueue.messages[i].address == address) {
                found = true;
            }
        }
        if (!found) {
            i++;
        }
    }
    if (!found) {
        i = locosMessageQueue.nextnewNMRA;
        if (i != (locosMessageQueue.division - 1)) {
            locosMessageQueue.nextnewNMRA = locosMessageQueue.nextnewNMRA + 1;
        } else {
            locosMessageQueue.nextnewNMRA = 0;
            //start again in 0
        };

        locosMessageQueue.messages[i].type = buffer[0];
        locosMessageQueue.messages[i].address = address;
        //cycling the queue
    }

    locosMessageQueue.messages[i].dspeed = buffer[2 + longadress];
    //speed and functions are stored on the same dspeed byte
    if ((buffer[0] == LOCODCCNfx) | (buffer[0] == LOCODCCLfx)) {
        locosMessageQueue.messages[i].fgroup = buffer[3 + longadress];
    }
}

void updateQueueMM() {
    char i = locosMessageQueue.division;
    static bit found;
    found = false;

    if ((buffer[0] == LOCOMM2Fx)|(buffer[0] == LOCOMM1R)) {
        buffer[0] = buffer[0] - 1; //look in main packet... we store two packets for LOCOMM2.. LOCOMM2 and LOCOMM2Fx
                                   //LOCOMM1R not to be soted... but the information of dspedd has to be updated.
    }

    while ((!found) && (i != MAXLOCOMESSAGES)) { //CYCLYINGqueue.. I need to check all
        if (locosMessageQueue.messages[i].type == buffer[0]) {
            if (locosMessageQueue.messages[i].address == buffer[1]) {
                found = true;
            }
        }
        if (!found) {
            i = i++; //we need to search 1 by one instead of two by two because MM only takes 1..
        }
    }
    if (!found) {
        i = locosMessageQueue.nextnewMM;
        locosMessageQueue.messages[i].type = buffer[0];
        locosMessageQueue.messages[i].address = buffer[1];
        locosMessageQueue.messages[i].dspeed = buffer[2];
        locosMessageQueue.messages[i].fgroup = 0b00000001 & buffer[3];
        //LOCOMM1R.. does not need anything else
        if (buffer[0] > LOCOMM1Fx) {
            locosMessageQueue.messages[i + 1].type = buffer[0] + 1;
            locosMessageQueue.messages[i + 1].address = buffer[1];
            locosMessageQueue.messages[i + 1].function = buffer[3];
            locosMessageQueue.messages[i + 1].fgroup = 1; //start refresh by the function 1
            locosMessageQueue.nextnewMM++;
        }
        if (locosMessageQueue.nextnewMM < (MAXLOCOMESSAGES - 2)) {          //always have two open positions... we never know next packet to arrive
            locosMessageQueue.nextnewMM = locosMessageQueue.nextnewMM + 1;  //it seems the one is worng.. but it is OK.. make a diagram in case of doubt
        } else {
            locosMessageQueue.nextnewMM = locosMessageQueue.division; //start again in 0
        };
    } else {
        locosMessageQueue.messages[i].dspeed = buffer[2];           
        locosMessageQueue.messages[i].fgroup = 0b00000001 & buffer[3];   //For MM1...in MM2 no really useful since everything is store in the next command
        if (buffer[0] > LOCOMM1Fx) {                                     //This is only valid because there is no definition on MM3, MM4 or MM5
            locosMessageQueue.messages[i + 1].function = buffer[3];
        }
    }
}

void updateQueueMMX() {
    char i = locosMessageQueue.division;
    static bit found;
    found = false;
    int address;

    address = buffer[2];
    byte_of(address, 1) = buffer[1];

    while ((!found) && (i != MAXLOCOMESSAGES)) { //CYCLYINGqueue.. I need to check all
        if (locosMessageQueue.messages[i].type == buffer[0]) {
            if (locosMessageQueue.messages[i].address == address) {
                found = true;
            }
        }
        if (!found) {
            i = i++; //we need to search 1 by one instead of two by two because MM only takes 1..
        }
    }
    if (!found) {
        i = locosMessageQueue.nextnewMM;
        locosMessageQueue.messages[i].type = buffer[0];
        locosMessageQueue.messages[i].address = address;
        locosMessageQueue.messages[i].dspeed = buffer[3];
        locosMessageQueue.messages[i + 1].type = buffer[0];
        locosMessageQueue.messages[i + 1].address = address;
        locosMessageQueue.messages[i + 1].function = buffer[4];
        locosMessageQueue.nextnewMM++;

        if (locosMessageQueue.nextnewMM < (MAXLOCOMESSAGES - 2)) {          //always have two open positions... we never know next packet to arrive
            locosMessageQueue.nextnewMM = locosMessageQueue.nextnewMM + 1;  //it seems the one is worng.. but it is OK.. make a diagram in case of doubt
        } else {
            locosMessageQueue.nextnewMM = locosMessageQueue.division;       //start again in 0
        };
    } else {
        locosMessageQueue.messages[i].dspeed = buffer[2];
        locosMessageQueue.messages[i + 1].function = buffer[3];
    }
}

void getfromQueueNMRA(char i) {
    static bit longadress;

    buffer[0] = locosMessageQueue.messages[i].type;
    if ((buffer[0] >= LOCODCCN14)&(buffer[0] <= LOCODCCNfx)) {
        //short address
        buffer[1] = locosMessageQueue.messages[i].address;          //We assume it was stored OK... no need to check again
        longadress = false;
    }
    if ((buffer[0] >= LOCODCCL14)&(buffer[0] <= LOCODCCLfx)) {
        buffer[1] = byte_of(locosMessageQueue.messages[i].address, 1);
        buffer[2] = byte_of(locosMessageQueue.messages[i].address, 0);
        longadress = true;
    }
    buffer[2 + longadress] = locosMessageQueue.messages[i].dspeed;
    if ((buffer[0] == LOCODCCNfx) | (buffer[0] == LOCODCCLfx)) {
        buffer[3 + longadress] = locosMessageQueue.messages[i].fgroup;
    }
}

void getfromQueueMM(char i) {

    buffer[0] = locosMessageQueue.messages[i].type;
    buffer[1] = locosMessageQueue.messages[i].address;          //We assume it was stored OK... no need to check again

    if (buffer[0] < LOCOMM2) {
        buffer[2] = locosMessageQueue.messages[i].dspeed;
        buffer[3] = locosMessageQueue.messages[i].fgroup;
    }
    if (buffer[0] == LOCOMM2) {
        buffer[2] = locosMessageQueue.messages[i].dspeed;
        buffer[3] = locosMessageQueue.messages[i + 1].function;
    }
    if (buffer[0] == LOCOMM2Fx) {
        buffer[2] = locosMessageQueue.messages[i - 1].dspeed;
        buffer[3] = locosMessageQueue.messages[i].function;
        buffer[4] = locosMessageQueue.messages[i].fgroup;
    }

    //For the alpha version of MMX
    if (buffer[0]== LOCOMMX){
        buffer[1] = byte_of(locosMessageQueue.messages[i].address, 1);
        buffer[2] = byte_of(locosMessageQueue.messages[i].address, 0);
        buffer[3] = locosMessageQueue.messages[i].dspeed;
        buffer[4] = locosMessageQueue.messages[i+1].function;
    }
}

char getMMoldDspeed(){

    char i = locosMessageQueue.division;
    static bit found;
    found = false;

    //it should not be necessary.. so far this function is only called when buffer[0]==LOCOMM1
    if (buffer[0] == LOCOMM2Fx) {
        buffer[0] = buffer[0] - 1; //look in main packet... we store two packets for LOCOMM2.. LOCOMM2 and LOCOMM2Fx
    }

    while ((!found) && (i != MAXLOCOMESSAGES)) { //CYCLYINGqueue.. I need to check all
        if (locosMessageQueue.messages[i].type == buffer[0]) {
            if (locosMessageQueue.messages[i].address == buffer[1]) {
                found = true;
            }
        }
        if (!found) {
            i = i++;    //we need to search 1 by one instead of two by two because MM only takes 1..
        }
    }
    if (!found){
        return 0;       //if the loco was not there no speed backwards, by default
    }else{
        return locosMessageQueue.messages[i].dspeed;
    }
}

char getMMoldFx(){

    char i = locosMessageQueue.division;
    static bit found;
    found = false;

    
    if (buffer[0] == LOCOMM2Fx) {
        buffer[0] = buffer[0] - 1; //look in main packet... we store two packets for LOCOMM2.. LOCOMM2 and LOCOMM2Fx
    }

    while ((!found) && (i != MAXLOCOMESSAGES)) { //CYCLYINGqueue.. I need to check all
        if (locosMessageQueue.messages[i].type == buffer[0]) {
            if (locosMessageQueue.messages[i].address == buffer[1]) {
                found = true;
            }
        }
        if (!found) {
            i = i++;         //we need to search 1 by one instead of two by two because MM only takes 1..
        }
    }
    if (!found){
        return 0;           //Everything off by default
    }else{
        if (buffer[0]==LOCOMM1){
            return locosMessageQueue.messages[i].fgroup;                //MM1FX not supported do far
        }else{
            return locosMessageQueue.messages[i+1].function;            //In theory.. only call if MM2
        }
    }
}

