/*
 * File:   serial.c
 * Author: Manolo Serrano
 * http://tren.enmicasa.net
 *
 * Created on 3 de marzo de 2013, 18:48
 *
 *  This file is part of rpic.
 *
 *  rpic is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  rpic is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with rpic.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Serial port routines
 * 
 */

/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */

#include "serial.h"
#include "timer.h"
#include "program.h"

/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

char bufferIN[MAXBYTES];

#if defined(PIC18)
#
#elif defined(PIC16)

void OpenUSART(unsigned char config, unsigned int spbrg) {
    SPBRGL = spbrg;
    SPBRGH=0;
    TXSTA = TXSTAconf;
    RCSTA = RCSTAconf;
    //I do not configure the INT here.. but in the initiINT.. different from PIC18
}

void CloseUSART() {
    SPEN = 0;           //DISABLE TRANSMISSION;
}

void baudUSART(unsigned char baudconfig) {
    //this is not implemented in PIC16 but it is impelmented inPIC16f1825
#ifdef __16F1825
    BAUDCON=baudconfig;
#endif
}

char ReadUSART(void) {
    return RCREG;
}

void WriteUSART(char data) {
    TXREG = data;
}
#endif

char checksum(char toverify[], char firstbyte, char lastbyte) {
    char i;
    char sum = toverify[firstbyte];

    for (i = firstbyte; i != lastbyte; i++) {
        sum = sum^toverify[i + 1];
    }
    return sum;
}

void bufferFirstByte() {
    bufferIN[0] = ReadUSART();
    char type = bufferIN[0]&0b00011111;
    char command = (bufferIN[0] >> 5)&0b00000111;
#if defined(PIC18)
    //NOT UPDATED WITH ALL POSSIBILITIES.. CHECK ALL POSSIBILITIES IN PIC16 IF BRACH HEREAFTER
    switch (command) {
        case SYSTEMCMD:
            commandlength = 2; //buffer0 buffer 1 buffer2
            break;
        case ACCNMRACMD:
            commandlength = 4;
            break;
        case ACCMMCMD:
            commandlength = 3;
            break;
        case LOCNMRACMD:
        case LOCMMCMD:
            switch (type) {
                case LOCOMM1R:
                    commandlength = 2;
                    break;
                case LOCOMM1Fx:
                case LOCODCCN14:
                case LOCODCCN28:
                case LOCODCCN128:
                    commandlength = 3;
                    break;
                case LOCOMM1:
                case LOCOMM2:
                case LOCOMM3:
                case LOCOMM4:
                case LOCOMM5:
                case LOCODCCNfx:
                case LOCODCCL14:
                case LOCODCCL28:
                case LOCODCCL128:
                    commandlength = 4;
                    break;
                case LOCOMM2Fx:
                case LOCOMM3Fx:
                case LOCOMM4Fx:
                case LOCOMM5Fx:
                case LOCODCCLfx:
                    commandlength = 5;
                    break;
            }
            break;
    }
#elif defined(PIC16)
    //PIC 16 does not take long swithces
    if (command == SYSTEMCMD) {
        commandlength = 2;
    };
    if (command == ACCNMRACMD) {
        commandlength = 4;
    };
    if (command == ACCMMCMD) {
        commandlength = 3;
    };
    if ((command == LOCNMRACMD) | (command == LOCMMCMD)| (command==LOCMMXCMD)) {
        switch (type) {
            case LOCOMM1R:
                commandlength = 2;
                break;
            case LOCOMM1Fx:
            case LOCODCCN14:
            case LOCODCCN28:
            case LOCODCCN128:
                commandlength = 3;
                break;
            case LOCOMM1:
            case LOCOMM2:
            case LOCOMM3:
            case LOCOMM4:
            case LOCOMM5:
            case LOCODCCNfx:
            case LOCODCCL14:
            case LOCODCCL28:
            case LOCODCCL128:
                commandlength = 4;
                break;
            case LOCOMM2Fx:
            case LOCOMM3Fx:
            case LOCOMM4Fx:
            case LOCOMM5Fx:
            case LOCODCCLfx:
            case LOCOMMX:                           //only for the alpha version of LOCOMMX normal versipon commande length=6
                commandlength = 5;
                break;
        }
    }

    if (command == PROGCMD) {
        switch (type) {
            case NMRA_SERVICE_MODE_ADDRESS_write:
            case NMRA_SERVICE_MODE_ADDRESS_read:
                commandlength = 2;
                break;
            case NMRA_SERVICE_MODE_REGISTER_write:
            case NMRA_SERVICE_MODE_REGISTER_read:
                commandlength = 3;
                break;
            case NMRA_SERVICE_MODE_DIRECT_write:
            case NMRA_SERVICE_MODE_DIRECT_verify:
            case NMRA_SERVICE_MODE_DIRECT_bit:
            case NMRA_SERVICE_MODE_PAGE_write:
            case NMRA_SERVICE_MODE_PAGE_read:
                commandlength = 4;
                break;
            case LOCODCCNwrite:
            case LOCODCNNverify:
            case LOCODCNNbit:
                commandlength = 5;
                break;
            case ACCNMRAwrite:
            case ACCNMRAverify:
            case ACCNMRAbit:
            case LOCODCCLwrite:
            case LOCODCNLverify:
            case LOCODCNLbit:
                commandlength=6;
                break;
        }
    }

    if (commandlength>0){bytesreceived=1;}
#endif
}

void bufferNextBytes() {
    bufferIN[bytesreceived] = ReadUSART();
    if (bytesreceived == commandlength) {
        if (status!=NEWCMD){
            old_status = status;    //avoid infinitive loop whnen two consequetive newcmd arrives
        }
        status = NEWCMD;
        if (bufferIN[commandlength] != checksum(bufferIN, 0, commandlength - 1)) {
            bufferIN[0] = ERROR1;   //Checksum error}....commandsystem=0.. type==error1=1
        }
        bufferIN[MAXBYTES - 1] = bufferIN[commandlength]; //In the last byte I always store the checkusm of the message for ID of message
        bytesreceived = 0;
        stopTimeout();
    } else {
        bytesreceived++;
    }
}
