/*
 * File:   user.c
 * Author: Manolo Serrano
 * http://tren.enmicasa.net
 *
 * Created on 26 de febrero de 2013, 9:51
 *
 *  This file is part of rpic.
 *
 *  rpic is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  rpic is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with rpic.  If not, see <http://www.gnu.org/licenses/>.
 *
 * User routines to init and configure PIC
 *
 */

/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/
#define USE_OR_MASKS

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#elif defined(__18CXX)
#include <p18cxxx.h>    /* C18 General Include File */
#endif

#if defined(__XC) || defined(HI_TECH_C)
#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */
#endif

#include "user.h"
#include "serial.h"
#include "queue.h"
#include "timer.h"


/******************************************************************************/
/* User Functions                                                             */
/******************************************************************************/

void InitApp(void) {
    // Setup analog functionality and port direction
    InitPort();
    // Initialize peripherals
    InitUSART();
    InitTIMER();
    // Enable interrupts
    InitInt();
    //Init Variables
    InitVar();
}

void InitPort(void) {

    PORTA = 0;
    TRISA = TRISAconf;
    
    PORTC = 0;
    TRISC = TRISCconf;


#if defined(PIC18)
#ifdef __18F14K22
    PORTB = 0;
    TRISB = TRISBconf;

    ADCON0 = ADCON0conf;
    ANSEL=ANSELconf;
    ANSELH = ANSELHconf;

    CM1CON0 = CM1CON0conf;
    CM2CON0 = CM2CON0conf;
    CM2CON1 = CM2CON1conf;
#endif
#elif defined(PIC16)
    /*    CMCON=7;  //for PIC16f6281
        PORTB=0;
        TRISB=TRISBconf;  */
    ANSELA=ANSELAconf;
    ANSELC=ANSELCconf;
    APFCON0=APFCON0conf;
#endif
}

void InitUSART() {
    CloseUSART();
    baudUSART(BAUDCONFIG);
    OpenUSART(CONFIGUSART, SPBRGconf);
}

void InitTIMER(void) {
    CloseTimer0();
#if defined(PIC18)
    //do nothing.. the opentimer Instruction do everything.. and we only want the timer to be on in specific moments
#elif defined(PIC16)
    OPTION_REG = OPTION_REGconf;
#endif
}

void InitInt(void) {
#if defined(PIC18)
    RCON = 0;   //no prioirities in interruption
    PIR1 = 0;
    PIR2 = 0;   //Clean up interruprs

    //In principle the int for uart and timer are also configuring their functions

    INTCON3 = INTCON3conf; //no interrupts needed from this registry
    INTCON2 = INTCON2conf; //no interrupts needed from this registry
    INTCON = INTCONconf;   //no interrupts needed from this registry

    PIE1 = PIE1conf;
    PIE2 = PIE2conf;

#elif defined(PIC16)
    INTCON = 0;
    PEIE = 1;    //periherials int
    TMR0IE=0;  //timer0 is always on.. i only want to send interuption when i charge the timer

    PIE1 = 0;
    RCIE = 1; //so far interrupt when receiving something
#endif
    ei();
}

void InitVar(void) {
    abortsignal = false;
    commandlength = 0;
    bytesreceived = 0;
    status = CYCLEOFF;

    initQueue();
}
