/*
 * File:   user.h
 * Author: Manolo Serrano
 * http://tren.enmicasa.net
 *
 * Created on 26 de febrero de 2013, 9:51
 *
 *  This file is part of rpic.
 *
 *  rpic is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  rpic is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with rpic.  If not, see <http://www.gnu.org/licenses/>.
 *
 * User definition of PIC configuration
 * 
 */

/******************************************************************************/
/* User Level #define Macros                                                  */
/******************************************************************************/

#define TRISAconf             0b00100000  //Ra5 Hi Impedance Z
#define TRISAout              0b00000000  //RA5 Z

#if defined(PIC18)
#define PinSignal             PORTAbits.RA5
#define TRISBconf             0b10100000  //RB7 and RB5 input (RX and TX) TRISB<1> and TRISB<2> should both be set to ?1? to configure the RB1/RX/DT and RB2/TX/CK pins as inputs. Output drive, when required, iscontrolled by the peripheral circuitry.
#define TRISCconf             0b00000000

#define INTCON3conf           0b00000000  //no need int here
#define INTCON2conf           0b00000000  //pull-up off.. no need int...
#define INTCONconf            0b01100000  //GIE is still off.. enabling it or not with ie() and di() /peripherial ON timer0on
#define PIE1conf              0b00100000  //Only interrupt on rx
#define PIE2conf              0b00000000

#define ADCON0conf            0b00000000   //NOADC enable
#define ANSELconf             0b00000000   //everything digital
#define ANSELHconf            0b00000000   //everything digital
#define CM1CON0conf           0b00000000   //everything digital
#define CM2CON0conf           0b00000000   //everything digital
#define CM2CON1conf           0b00000000   //everything digital

#elif defined(PIC16)
#define PinSignal             RA5
#define TRISCconf             0b00110000  //RC5 and RC4 input (RX and TX) 
#define APFCON0conf           0b00000000
#define ANSELAconf            0b00000000
#define ANSELCconf            0b00000000
#endif

//coming from manual and http://www.microchip.com/forums/m701812-print.aspx

#define testbit(var, bit)   ((var) & (1 <<(bit)))
#define setbit(var, bit)    ((var) |= (1 << (bit)))
#define clrbit(var, bit)    ((var) &= ~(1 << (bit)))
#define bittgl(var,bit)     ((var) ^= (1<<(bit)))

#define byte_of(your_var,byte_num) (*(((unsigned char*)&your_var)+byte_num))

/******************************************************************************/
/* User Function Prototypes                                                   */
/******************************************************************************/

void InitApp(void);         /* I/O and Peripheral Initialization */
void InitUSART(void);
void InitTIMER(void);
void InitInt(void);         /*Init interruptions*/
void InitPort(void);        /*Initi port*/
void InitVar(void);         /*Initi port*/
